import argparse, traceback, sys

from car_framework.context import Context, context
from car_framework.util import IncrementalImportNotPossible, RecoverableFailure, UnrecoverableFailure


class BaseApp(object):
    def __init__(self, description):
        self.parser = argparse.ArgumentParser(description=description)
        self.parser.add_argument('-car-service-url', dest='car_service', type=str, required=True, help='URL of the CAR ingestion service')
        self.parser.add_argument('-car-service-key', dest='api_key', type=str, required=True, help='API key for CAR ingestion service')
        self.parser.add_argument('-car-service-password', dest='api_password', type=str, required=True, help='Password for CAR ingestion service')

        # source id to uniquely identify each data source
        self.parser.add_argument('-source', dest='source', type=str, required=True, help='Unique source id for the data source')

        self.parser.add_argument('-d', dest='debug', action='store_true', help='Enables DEBUG level logging')


    def setup(self):
        args = self.parser.parse_args()
        Context(args)


    def run(self):
        try:
            try:
                context().logger.info('Attempting incremental import...')
                context().inc_importer.run()
            except IncrementalImportNotPossible as e:
                context().logger.info('Attempting full import...')
                context().full_importer.run()

            context().logger.info('Done.')

        except RecoverableFailure as e:
            context().logger.info('Recoverable failure: ' + e.message)
            context().logger.info('Incremental import will be attempted again in the nex run.')
            sys.exit(1)
        except UnrecoverableFailure as e:
            context().logger.info('Unrecoverable failure: ' + e.message)
            context().logger.info('Incremental import will not be possible in the nex run.')
            context().car_service.reset_model_state_id()
            sys.exit(1)
        except Exception as e:
            context().logger.exception(e)
            context().logger.error(traceback.format_exc())
            # traceback.print_exc()
            sys.exit(1)



import logging
from pythonjsonlogger import jsonlogger
from datetime import datetime

class CustomJsonFormatter(jsonlogger.JsonFormatter):
    def add_fields(self, log_record, record, message_dict):
        # use inherited constructor
        super(CustomJsonFormatter, self).add_fields(
            log_record, record, message_dict)
        if not log_record.get('ibm_datetime'):
            now = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S.%fZ')
            log_record['ibm_datetime'] = now

        # assign values to log_record
        log_record['level'] = log_record['level'].lower(
        ) if log_record.get('level') else record.levelname
        log_record['message'] = log_record['message'] if log_record.get(
            'log') else record.message
        log_record['label'] = log_record['label'] if log_record.get(
            'type') else record.name

def create_logger(debug = False):
    logger = logging.getLogger()
    logger.setLevel(debug and logging.DEBUG or logging.INFO)

    handler = logging.StreamHandler()
    handler.setLevel(debug and logging.DEBUG or logging.INFO)
    format_string = '(ibm_datetime) (level) (label) (message)'
    formatter = CustomJsonFormatter(format_string)
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    return logger


class Context(object):
    def __init__(self, args):
        global global_context
        global_context = self

        from car_framework.car_service import CarService
        from car_framework.communicator import Communicator
        self.args = args
        self.logger = create_logger(args.debug)
        self.car_service = CarService(Communicator())
        

global_context = None
def context():
    return global_context



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="car-connector-framework",
    version="0.0.16",
    author="IBM",
    author_email="",
    description="CAR service connector framework",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/IBM/cp4s-car-connector-framework",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'requests>=2.24.0', # This is for prod pypi
        # 'requests', # This is for test pypi
        'python-json-logger==0.1.9',
    ],
    license='Apache License 2.0',
    platforms=["Any"]
)